import getUrlContents from '../getUrlContents';
import { Product } from '../interfaces/ISettings';
import verifyProductName from '../verifyProductName';
import extractFile from './extractFile';
import readSsnFile from './readSsnFile';

export default async function getSolidpkg(product: Product, from: number, to: number): Promise<ArrayBuffer> {
  //Verify function arguments
  if (!verifyProductName(product)) {
    throw new Error(`"${product}" is not a valid product.`);
  }
  if (typeof from !== 'number' || (from | 0) !== from || from < -1) {
    throw new Error(`from must be an integer greater than or equal to -1 but it is "${from}"`);
  }
  if (typeof to !== 'number' || (to | 0) !== to || to < 0) {
    throw new Error(`to must be an integer greater than or equal to 0 but it is "${to}"`);
  }
  if (from >= to) {
    throw new Error(`from must be less than to but "${from}" is not less than "${to}".`);
  }

  //Download .solidpkg file
  const ssnFile = await getUrlContents({ host: 'cdn-patch.swtor.com', path: `/patch/${product}/${product}_${from}to${to}.solidpkg` });

  //Parse .solidpkg file
  const fileEntries = readSsnFile(ssnFile);

  if (fileEntries.length !== 1) {
    throw new Error(`Expected .solidpkg to contain 1 file but it had "${fileEntries.length}" files.`);
  }

  const firstFile = fileEntries[0];
  if (firstFile.name !== 'metafile.solid') {
    throw new Error(`Expected .solidpkg to contain a file called metafile.solid but it is called "${firstFile.name}".`);
  }

  //Extract metafile.solid file
  const solidFile = await extractFile(firstFile, [new DataView(ssnFile)]);

  return solidFile;
}
